import { ColumnInstance } from 'react-table';
import React from 'react';
import AppSelectField from 'TextField/AppSelectField';
import { IconButton, InputAdornment } from '@material-ui/core';
import ClearIcon from '@material-ui/icons/Clear';
import styles from './SelectColumnFilter.module.scss';
import clsx from 'clsx';

type Props = {
  column: ColumnInstance<any>;
  options: Record<string, any>;
  title?: string;
  valueKey?: string;
  labelKey?: string;
};

function SelectColumnFilter(props: Props) {
  const { column, title, options, ...otherProps } = props;
  const { filterValue, setFilter, Header, id } = column;
  return (
    <AppSelectField
      filterValue={filterValue}
      setFilter={setFilter}
      options={options}
      placeholder={'ALL'}
      label={title || (Header as string) || id}
      classes={{
        root: styles.inputBase,
      }}
      InputProps={{
        endAdornment: filterValue && (
          <InputAdornment className={clsx(styles.inputAdorned)} position="end">
            <IconButton
              className={styles.inputIcon}
              onClick={() => setFilter(undefined)}
              title={'Clear'}
              tabIndex={-1}
            >
              <ClearIcon />
            </IconButton>
          </InputAdornment>
        ),
      }}
      {...otherProps}
    />
  );
}

export default SelectColumnFilter;
