import React from 'react';
import { createSelectable } from 'react-selectable-fast';
import { selectSingleRow } from 'store/actions';
import clsx from 'clsx';
import styles from 'EnhancedTable.module.scss';
import { Row } from 'react-table';
import { useDispatch } from 'react-redux';

type Props = {
  row: Row<any>;
  previousRow: Row | null;
  setPreviousRow: (value: Row | null) => void;
  toggleAllRowsSelected: (value?: boolean) => void;
  rows: Array<Row<any>>;
};

const RowSelectable = createSelectable<Props>((props) => {
  const dispatch = useDispatch();
  const {
    selectableRef,
    row,
    previousRow,
    setPreviousRow,
    toggleAllRowsSelected,
    rows,
  } = props;

  function toggleCurrentRow() {
    row.toggleRowSelected();
    setPreviousRow(row);
  }

  function clickHandler(event: React.MouseEvent) {
    if (event.ctrlKey) {
      toggleCurrentRow();
      return;
    }

    if (event.shiftKey) {
      shiftKeyHandler();
      return;
    }

    // @ts-ignore
    const tagName = event.target.tagName;
    if (['TD', 'TR', 'DIV', 'SPAN'].includes(tagName)) {
      toggleAllRowsSelected(false);
      toggleCurrentRow();
    }
  }

  function shiftKeyHandler() {
    if (previousRow) {
      toggleAllRowsSelected(false);
      rows.forEach((pageRow) => {
        const inLower =
          Number(previousRow.id) <= Number(pageRow.id) &&
          Number(pageRow.id) <= Number(row.id);
        const inUpper =
          Number(previousRow.id) >= Number(pageRow.id) &&
          Number(pageRow.id) >= Number(row.id);
        if (inLower || inUpper) {
          pageRow.toggleRowSelected();
        }
      });
    } else {
      toggleCurrentRow();
    }
  }

  return (
    <div
      onClick={clickHandler}
      onContextMenu={(event) => {
        row.toggleRowSelected(true);
        dispatch(selectSingleRow(row.original));
      }}
      {...row.getRowProps()}
      className={clsx(styles.tr, row.isSelected && styles.selected)}
      ref={selectableRef}
    >
      {row.cells
        .filter((cell) => !cell.column.isHidden)
        .map((cell) => {
          return (
            <div className={styles.td} {...cell.getCellProps()}>
              {cell.render('Cell')}
            </div>
          );
        })}
    </div>
  );
});

export default RowSelectable;
