import React, { FunctionComponent, useCallback, useRef, useState } from 'react';
import MuiTheme from 'MuiTheme';
import TodayIcon from '@material-ui/icons/Today';
import AppTextField from 'TextField/AppTextField';
import { DateTimePicker, DateTimePickerProps } from '@material-ui/pickers';
import ClearIcon from '@material-ui/icons/Clear';
import { getDatabaseFormatDateTime } from 'utils/moment';
import { IconButton } from '@material-ui/core';
import DoneRoundedIcon from '@material-ui/icons/DoneRounded';
import styles from './EditableCell.module.scss';
import tableStyles from '../EnhancedTable.module.scss';
import EditIcon from '@material-ui/icons/Edit';
import { useOnClickOutside } from 'utils/customHook';
import { useDispatch } from 'react-redux';
import { setTableSelectable } from '../store/actions';

type Props = Partial<DateTimePickerProps> & {
  inputTimeFormat?: string;
  row: any;
};

const EditableDateTime: FunctionComponent<Props> = (props) => {
  const dispatch = useDispatch();
  const setSelectable = useCallback(
    (value: boolean) => {
      dispatch(setTableSelectable(value));
    },
    [dispatch]
  );
  const { row, inputTimeFormat } = props;

  const divEl = useRef<HTMLInputElement>(null);
  useOnClickOutside(divEl, () => cancelTimePicker());

  const [editDate, setEditDate] = useState<boolean>(false);
  const [selectedDate, setSelectedDate] = useState<Date | any>(row.value);

  function openTimePicker() {
    setEditDate(true);
    setSelectable(true);
  }

  function cancelTimePicker() {
    setSelectedDate(row.value);
    setEditDate(false);
  }

  function updateData() {
    if (row.column.editData) {
      row.column.editData(row.row.original, selectedDate, row.column.id);
    }
    setEditDate(false);
  }

  return (
    <div className={styles.formEditableDateTime}>
      {!editDate ? (
        <span>
          {getDatabaseFormatDateTime(selectedDate as string)}
          <IconButton
            className={tableStyles.displayOnHover}
            size="small"
            onClick={openTimePicker}
          >
            <EditIcon color="primary" fontSize="small" />
          </IconButton>
        </span>
      ) : (
        <DateTimePicker
          value={selectedDate}
          onChange={(newVal) => {
            setSelectedDate(getDatabaseFormatDateTime(newVal as string));
          }}
          desktopModeMediaQuery={MuiTheme.breakpoints.up('sm')}
          openPickerIcon={<TodayIcon color={'secondary'} />}
          inputFormat={inputTimeFormat}
          ampmInClock={true}
          clearable={true}
          renderInput={(props) => {
            return (
              <AppTextField
                {...props}
                InputLabelProps={{ shrink: true }}
                InputProps={{
                  ...props.InputProps,
                  endAdornment: (
                    <div className={styles.formButton}>
                      {props.InputProps?.endAdornment}
                      <IconButton size="small" onClick={updateData}>
                        <DoneRoundedIcon color="primary" fontSize="small" />
                      </IconButton>
                      <IconButton
                        size="small"
                        onClick={() => cancelTimePicker()}
                      >
                        <ClearIcon color="primary" fontSize="small" />
                      </IconButton>
                    </div>
                  ),
                }}
                helperText={false}
              />
            );
          }}
        />
      )}
    </div>
  );
};

export default EditableDateTime;
