import React, { FunctionComponent, useMemo } from 'react';
import TablePaginationActions from 'Pagination/TablePaginationActions';
import styles from 'Pagination/ReactTablePagination.module.scss';
import AppTextField from 'TextField/AppTextField';
import { Autocomplete } from '@material-ui/lab';

type Props = {
  gotoPage: (page: number) => void;
  setPageSize: (page: number) => void;
  pageIndex: number;
  pageSize: number;
  pageTotal: number;
};

type Option = {
  label: string;
  value: number;
  inputValue?: string;
};
const ReactTablePagination: FunctionComponent<Props> = (props) => {
  const { gotoPage, setPageSize, pageIndex, pageSize, pageTotal } = props;
  const firstRow = pageIndex * pageSize + 1 || 0;
  const lastRow = Math.min(pageIndex * pageSize + pageSize, pageTotal) || 0;
  const pageOptions = useMemo(() => {
    const defaultOptions = [
      { label: '5', value: 5 },
      { label: '10', value: 10 },
      { label: '25', value: 25 },
      { label: '50', value: 50 },
      { label: 'All', value: pageTotal },
    ];
    if (defaultOptions.every((item) => item.value !== pageSize)) {
      defaultOptions.unshift({
        label: pageSize.toString(),
        value: pageSize,
      });
    }
    return defaultOptions;
  }, [pageSize, pageTotal]);

  const initPageSize = {
    label: pageSize.toString(),
    value: pageSize,
  };
  const handleChangePage = (
    event: React.MouseEvent<HTMLButtonElement> | null,
    page: number
  ) => {
    gotoPage(page);
  };

  return pageTotal ? (
    <span className={styles.pagination}>
      <span className={styles.inputForm}>
        <span>Show</span>:{' '}
        <Autocomplete<Option, false, true, true>
          value={initPageSize}
          onChange={(event, newValue) => {
            const newSize =
              typeof newValue === 'string' ? newValue : newValue.value;
            setPageSize(Number(newSize));
          }}
          filterOptions={(options, params) => {
            const filtered = [...options];
            // Suggest the creation of a new value
            if (
              params.inputValue !== '' &&
              options.every(
                (item) => Number(item.label) !== Number(params.inputValue)
              )
            ) {
              filtered.unshift({
                value: Number(params.inputValue),
                inputValue: params.inputValue,
                label: params.inputValue,
              });
            }

            return filtered;
          }}
          getOptionLabel={(option) => option?.label || ''}
          disableClearable={true}
          handleHomeEndKeys
          blurOnSelect
          clearOnBlur={true}
          selectOnFocus={true}
          freeSolo={true}
          options={pageOptions}
          renderOption={(option) => option.label}
          renderInput={(params) => <AppTextField {...params} />}
          classes={{
            option: styles.selectOption,
          }}
        />
      </span>
      <span>
        {firstRow}-{lastRow} of {pageTotal}
      </span>
      <TablePaginationActions
        count={pageTotal || 0}
        getItemAriaLabel={() => ''}
        onChangePage={handleChangePage}
        page={pageIndex}
        rowsPerPage={pageSize}
        showFirstButton
        showLastButton
      />
    </span>
  ) : (
    <span />
  );
};

export default ReactTablePagination;
