import React, { Fragment, FunctionComponent, useState } from 'react';
import styles from 'EnhancedTable.module.scss';
import { Popover } from '@material-ui/core';
import { AppActionIcon } from 'assets/icons/AppIcon';
import { TableActionMenuProvider } from 'Toolbar/TableActionMenuContext';
import clsx from 'clsx';

const TableActionBtn: FunctionComponent = (props) => {
  const [anchorEl, setAnchorEl] = React.useState<HTMLButtonElement | null>(
    null
  );
  const [hidden, setHidden] = useState(false);

  const handleClick = (event: React.MouseEvent<HTMLButtonElement>) => {
    setAnchorEl(event.currentTarget);
    setHidden(false);
  };

  const handleHidden = () => {
    setHidden(true);
  };
  const handleClose = () => {
    setAnchorEl(null);
  };
  const open = Boolean(anchorEl);

  return (
    <Fragment>
      <span className={styles.action} onClick={handleClick}>
        <AppActionIcon className={styles.iconAction} />
        <span>ACTIONS</span>
      </span>
      <Popover
        className={clsx(hidden && 'hidden')}
        open={open}
        anchorEl={anchorEl}
        onClose={handleClose}
        anchorOrigin={{
          vertical: 'bottom',
          horizontal: 'left',
        }}
        transformOrigin={{
          vertical: 'top',
          horizontal: 'left',
        }}
      >
        <div>
          <TableActionMenuProvider
            value={{
              handleClose: handleHidden,
              close: open,
            }}
          >
            {props.children}
          </TableActionMenuProvider>
        </div>
      </Popover>
    </Fragment>
  );
};

export default TableActionBtn;
